%Shell program for the example given in Secition 6.3 of Arcidiacono and
%Miller "Nonstationary Dynamic Models with Finite Dependence"
%This program solves for the finite dependence path given the states of two
%players and the time period.
%In lines 16-29 users can choose the following:
%1) The state of the two players and the time period
%2) The dimensions of the problem (numbert of states and time periods)
%3) The probability of nature resetting the state (given by pi)
%4) The payoff parameters
%
%The output of the program is Choicetplus2 and Choicetplus3 which gives the finited dependence 
%weights in periods t+2 and t+3  

clear
clc

global sol sol2 j1 j2

%fstate gives initial state of player i and player j where we're trying to get finite dependence for player i
%t0 gives the period where you're checking finite dependence. Note that
%t0 must be an integer less than T-3 and greater than 0
fstate=[4;5]; 
t0=1;

T=20;           %Number of Time periods
Smin=2;         %Min of each player's state 
Smax=25;        %Max of each player's state 
pi=.05;         %Exogenous probability of the system resetting

%Payoff parameters

b1=.35;         %coefficient on own log state
b2=-.15;        %coefficient on average of other players log state * log of own state
b3=-3;          %cost of changing the state
Beta=.90;       %discount factor

%First get fixed point in the last period
%P and Po refer to choice probabilities given the states, iterating on
%these until they are the same. Note that, because of the way these are
%constructed, the transpose of the P matrix gives the probabilities for the
%other player
%X1 (X2) is the resulting own state from choosing action 1 (2). Similar for
%the states resulting from the other player's choices (Xo1 and Xo2)

P=ones(Smax-Smin+1,Smax-Smin+1);
Po=zeros(Smax-Smin+1,Smax-Smin+1);

X1=[Smin:Smax]';
X2=[[Smin+1:Smax] Smax]';
Xo1=[Smin:Smax];
Xo2=[[Smin+1:Smax] Smax];

Xo1=ones(Smax-Smin+1,1)*Xo1;
Xo2=ones(Smax-Smin+1,1)*Xo2;
j=1;


while max(max(abs(P-Po)))>.00000001
    Po=P;
    temp=(1-Po').*log(Xo1)+Po'.*log(Xo2);
    
    U2=(log(X2)*ones(1,Smax-Smin+1))*(b1+b2*temp)+b3;
    U1=(log(X1)*ones(1,Smax-Smin+1))*(b1+b2*temp);
    
    P=exp(U2-U1)./(exp(U2-U1)+1);
        
    j=j+1;
    
end

%In the second to last period, players have expectations over future utilty. With
%some probabiliy pi both players get knocked down to the minimum state

PT=zeros(Smax-Smin+1,Smax-Smin+1,T);
PT(:,:,T)=P;

P=ones(Smax-Smin+1,Smax-Smin+1);
Po=.5*ones(Smax-Smin+1,Smax-Smin+1);


v2=U2;
v1=U1;

Vtemp=Beta*log(exp(v2)+exp(v1));
VH=[Vtemp Vtemp(:,Smax-Smin+1)];
VH=[VH;VH(Smax-Smin+1,:)];

VL=[Vtemp(1:end-1,1) Vtemp(1:end-1,1:end-1) Vtemp(1:end-1,end-1)];
VL=[VL(1,:);VL;VL(end,:)];

j=1;

t=T-1;

while t>0

    while  max(max(abs(P-Po)))>.00000001
        Po=P;
        temp=(1-Po').*log(Xo1)+Po'.*log(Xo2);
    
        Fv2=Po'.*((1-pi)*VH(2:end,2:end)+pi*VH(1,1))+(1-Po').*((1-pi)*VH(2:end,1:end-1)+pi*VH(1,1));
        Fv1=Po'.*((1-pi)*VH(1:end-1,2:end)+pi*VH(1,1))+(1-Po').*((1-pi)*VH(1:end-1,1:end-1)+pi*VH(1,1));
    
        U2=(log(X2)*ones(1,Smax-Smin+1))*(b1+b2*temp)+b3+Fv2;
        U1=(log(X1)*ones(1,Smax-Smin+1))*(b1+b2*temp)+Fv1;
    
        P=exp(U2-U1)./(exp(U2-U1)+1);
        
    end

    Vtemp=Beta*log(exp(U2)+exp(U1));
    VH=[Vtemp Vtemp(:,Smax-Smin+1)];
    VH=[VH;VH(Smax-Smin+1,:)];

    PT(:,:,t)=P;
    P=ones(Smax-Smin+1,Smax-Smin+1);
    Po=zeros(Smax-Smin+1,Smax-Smin+1);

t=t-1;

end


%lining up the finite dependence path for the case where nature does not move at t+1 or t+2 as trivially
%lined up when nature moves
%Max possible competitor states at t+3 is 4: choose 2 at t=t0, t=t0+1, t=t0+2, up the
%state by 3 units plus could have done nothing resulting in the state
%staying the same
%Max combination of states at t+2 is 9: all combinations of 3 states for
%each player
%Ordering of Ptrans is ownstate 1:3, comp state 1, ownstate 1:3, comp state
%3, etc.
%Note that this effectively matches that state at t+3, after the t+3 move
%but before nature moves at t+3


Ptrans=zeros(9,4);

j=1;
while j<4
    k=1;
    while k<4
        l=1;
        while l<5
            if l==k
                Ptrans(j+(k-1)*3,l)=1-PT(fstate(2)+k-1,fstate(1)+j-1,t0+2);
            end
            if l==(k+1)
                Ptrans(j+(k-1)*3,l)=PT(fstate(2)+k-1,fstate(1)+j-1,t0+2);
            end
            l=l+1;
        end
        k=k+1;
    end
    j=j+1;
end

%now getting transition matrix from t+1 states to t+2 states given the
%initial decision. Have 9 t+2 states and 2 t+1 states (2 t+1 states for
%each initial decision)
%order of 9 t+2 states has to match order of states for PTrans

F22=zeros(2,9);
F21=F22;
F12=F22;
F11=F22;

F22(1,3)=1-PT(fstate(2),fstate(1)+1,t0+1);
F22(1,6)=PT(fstate(2),fstate(1)+1,t0+1);
F22(2,6)=1-PT(fstate(2)+1,fstate(1)+1,t0+1);
F22(2,9)=PT(fstate(2)+1,fstate(1)+1,t0+1);
F12(1,2)=F22(1,3);
F12(1,5)=F22(1,6);
F12(2,5)=F22(2,6);
F12(2,8)=F22(2,9);

F21(1,2)=1-PT(fstate(2),fstate(1),t0+1);
F21(1,5)=PT(fstate(2),fstate(1),t0+1);
F21(2,5)=1-PT(fstate(2)+1,fstate(1),t0+1);
F21(2,8)=PT(fstate(2)+1,fstate(1),t0+1);
F11(1,1)=F21(1,2);
F11(1,4)=F21(1,5);
F11(2,4)=F21(2,5);
F11(2,7)=F21(2,8);

K11=[1-PT(fstate(2),fstate(1),t0);PT(fstate(2),fstate(1),t0)];
K22=K11;

%in order to line up the states for firm1, know that must only have 1 or 2
%more stores as those are the ones that are feasible along both paths.
%Hence enter,enter must be followed by doing nothing and nothing,nothing
%must be followed by enter. also, conditional on this, can fix the weights
%for nothing,enter and only estimate the weights for enter,nothing

BF22=zeros(3,8);
BF12=zeros(3,8);
BF21=zeros(3,8);
BF11=zeros(3,8);

BF11(1,1)=Ptrans(2,1);
BF11(1,3)=Ptrans(2,2);
BF11(2,3)=Ptrans(5,2);
BF11(2,5)=Ptrans(5,3);
BF11(3,5)=Ptrans(8,3);
BF11(3,7)=Ptrans(8,4);

BF21(1,2)=Ptrans(2,1);
BF21(1,4)=Ptrans(2,2);
BF21(2,4)=Ptrans(5,2);
BF21(2,6)=Ptrans(5,3);
BF21(3,6)=Ptrans(8,3);
BF21(3,8)=Ptrans(8,4);

BF12(1,1)=Ptrans(2,1);
BF12(1,3)=Ptrans(2,2);
BF12(2,3)=Ptrans(5,2);
BF12(2,5)=Ptrans(5,3);
BF12(3,5)=Ptrans(8,3);
BF12(3,7)=Ptrans(8,4);

BF22(1,2)=Ptrans(2,1);
BF22(1,4)=Ptrans(2,2);
BF22(2,4)=Ptrans(5,2);
BF22(2,6)=Ptrans(5,3);
BF22(3,6)=Ptrans(8,3);
BF22(3,8)=Ptrans(8,4);

%decision when entered twice is to do nothing so need to get the
%transitions given this choice

FF12=zeros(3,8);
FF12(1,2)=Ptrans(3,1);
FF12(1,4)=Ptrans(3,2);
FF12(2,4)=Ptrans(6,2);
FF12(2,6)=Ptrans(6,3);
FF12(3,6)=Ptrans(9,3);
FF12(3,8)=Ptrans(9,4);

%decsision when never entered is to always enter so need those transitions

FF21=zeros(3,8);
FF21(1,1)=Ptrans(1,1);
FF21(1,3)=Ptrans(1,2);
FF21(2,3)=Ptrans(4,2);
FF21(2,5)=Ptrans(4,3);
FF21(3,5)=Ptrans(7,3);
FF21(3,7)=Ptrans(7,4);

%setting up the inputs to the fuction that searches for the knife edge at period t+1 so all the states
%will line up after the t+2 decision. note all inversions are done outside
%of the optimization routine.

rhs=Ptrans(:,1:3)'*[F11;-F12]'*[K11;K22];
inv1=Ptrans(:,1:3)'*[F22-F12;F11-F21]';
inv11=inv(inv1(:,2:4));
F1b=inv1(:,1);

minv2=[BF22-BF12];
minv22=[minv2(:,2) minv2(:,4) minv2(:,8)];
inv2=inv(minv22');

[bb,like]=fminunc('likelag2',0,[],rhs,inv11,K22,K11,F1b,F11,F12,F22,F21,inv2,BF11,BF12,BF22,FF12,FF21);

%The elements of Choicetplus2 are the weights on raising the state. The
%first two elements are when player i raised the state in period t while
%player j did not and did respectively.
%third and fourth elements are when player i did not raise the state in
%period t while player j did not and did respectively

Choicetplus2=sol(:,1);

%The elements of Choicetplus3 are the weights on raising the state along
%the path where player i raised the state at t but did not raise the
%state at t+1. All other weights are fixed at the levels given in Table 1
%of the paper

Choicetplus3=sol2;

%The first State shows the probability of each of the states at period t+3 given the
%intial choices of not raising the state (column1) and raising the state
%(column 2) to confirm that the distribution of states match across the two
%paths
State=[j1 j2];



%save finalsol Choicetplus2 Choicetplus3 State fstate t0 like

